/**
The MIT License (MIT)

Copyright (c) 2021 Martin Heidegger
Copyright (c) 2022 dao.xyz

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 */
export declare function fromBits(low: any, high: any, unsigned: any, target?: any): any;
export declare class Timestamp {
    wallTime: bigint;
    logical: number;
    constructor(properties?: {
        wallTime: bigint;
        logical?: number;
    });
    static compare(a: Timestamp, b: Timestamp): 1 | 0 | -1;
    static bigger(a: Timestamp, b: Timestamp): Timestamp;
    static smaller(a: Timestamp, b: Timestamp): Timestamp;
    compare(other: Timestamp): 1 | 0 | -1;
    clone(): Timestamp;
    toString(): string;
}
export declare class HLC {
    maxOffset: bigint;
    wallTimeUpperBound: bigint;
    toleratedForwardClockJump: bigint;
    last: Timestamp;
    wallTime: () => bigint;
    constructor(properties?: {
        wallTime?: () => bigint;
        maxOffset?: bigint;
        wallTimeUpperBound?: bigint;
        toleratedForwardClockJump?: bigint;
        last?: Timestamp;
    });
    now(): Timestamp;
    validateOffset(offset: bigint): void;
    update(other: Timestamp): Timestamp;
}
export declare class ClockOffsetError extends Error {
    offset: bigint;
    maxOffset: bigint;
    constructor(offset: bigint, maxOffset: bigint);
}
export declare class WallTimeOverflowError extends Error {
    time: bigint;
    maxTime: bigint;
    constructor(time: bigint, maxTime: bigint);
}
export declare class ForwardJumpError extends Error {
    timejump: bigint;
    tolerance: bigint;
    constructor(timejump: bigint, tolerance: bigint);
}
export declare class LamportClock {
    id: Uint8Array;
    timestamp: Timestamp;
    constructor(properties?: {
        id: Uint8Array;
        timestamp?: Timestamp | number;
    });
    clone(): LamportClock;
    equals(other: LamportClock): boolean;
    /**
     * Not optimized, dont use for performance critical things
     * @returns
     */
    advance(): LamportClock;
    static compare(a: LamportClock, b: LamportClock): number;
}
