import { Entry } from "./entry.js";
import { PublicKeyEncryptionResolver } from "@dao-xyz/peerbit-crypto";
import { Encoding } from "./encoding.js";
import { BlockStore } from "@dao-xyz/libp2p-direct-block";
export interface EntryFetchOptions<T> {
    length?: number;
    timeout?: number;
    onFetched?: (entry: Entry<T>) => void;
    concurrency?: number;
    encoding?: Encoding<T>;
    encryption?: PublicKeyEncryptionResolver;
}
interface EntryFetchStrictOptions<T> {
    length: number;
    timeout?: number;
    onFetched?: (entry: Entry<T>) => void;
    concurrency: number;
    encoding?: Encoding<T>;
    encryption?: PublicKeyEncryptionResolver;
}
export interface EntryFetchAllOptions<T> extends EntryFetchOptions<T> {
    shouldFetch?: (string: string) => boolean;
    shouldQueue?: (string: string) => boolean;
    onFetch?: (hash: string) => void;
    onQueueCallback?: (hash: string) => void;
    cache?: Map<string, Entry<any>>;
    delay?: number;
}
interface EntryFetchAllStrictOptions<T> extends EntryFetchStrictOptions<T> {
    shouldFetch?: (string: string) => boolean;
    shouldQueue?: (string: string) => boolean;
    onQueueCallback?: (hash: string) => void;
    onFetch?: (hash: string) => void;
    cache?: Map<string, Entry<any>>;
    delay: number;
}
export declare const strictAllFetchOptions: <T>(options: EntryFetchAllOptions<T>) => EntryFetchAllStrictOptions<T>;
export declare const strictFetchOptions: <T>(options: EntryFetchOptions<T>) => EntryFetchStrictOptions<T>;
export declare class EntryIO {
    static fetchParallel<T>(store: BlockStore, hashes: string | string[], options: EntryFetchAllOptions<T>): Promise<Entry<T>[]>;
    static fetchAll<T>(store: BlockStore, hashes: string | string[], fetchOptions: EntryFetchAllOptions<T>): Promise<Entry<T>[]>;
}
export {};
