var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Entry_1;
import { HLC, LamportClock as Clock, Timestamp } from "./clock.js";
import { isDefined } from "./is-defined.js";
import { variant, field, serialize, deserialize, option, vec, fixedArray, } from "@dao-xyz/borsh";
import { DecryptedThing, MaybeEncrypted, X25519PublicKey, SignatureWithKey, AccessError, Ed25519PublicKey, sha256Base64, randomBytes, } from "@dao-xyz/peerbit-crypto";
import { verify } from "@dao-xyz/peerbit-crypto";
import { arraysCompare, arraysEqual } from "@dao-xyz/peerbit-borsh-utils";
import { JSON_ENCODING } from "./encoding.js";
import { StringArray } from "./types.js";
import { logger } from "./logger.js";
import { createBlock, getBlockValue } from "@dao-xyz/libp2p-direct-block";
const isMaybeEryptionPublicKey = (o) => {
    if (!o) {
        return true;
    }
    if (o instanceof X25519PublicKey || o instanceof Ed25519PublicKey) {
        return true;
    }
    if (Array.isArray(o)) {
        return true; // assume entries are either X25519PublicKey or Ed25519PublicKey
    }
    return false;
};
function arrayToHex(arr) {
    return [...new Uint8Array(arr)]
        .map((b) => b.toString(16).padStart(2, "0"))
        .join("");
}
export function toBufferLE(num, width) {
    const hex = num.toString(16);
    const padded = hex.padStart(width * 2, "0").slice(0, width * 2);
    const arr = padded.match(/.{1,2}/g)?.map((byte) => parseInt(byte, 16));
    if (!arr) {
        throw new Error("Unexpected");
    }
    const buffer = Uint8Array.from(arr);
    buffer.reverse();
    return buffer;
}
export function toBigIntLE(buf) {
    const reversed = buf.reverse();
    const hex = arrayToHex(reversed);
    if (hex.length === 0) {
        return BigInt(0);
    }
    return BigInt(`0x${hex}`);
}
let Payload = class Payload {
    data;
    _value;
    constructor(props) {
        if (props) {
            this.data = props.data;
            this._value = props.value;
        }
    }
    equals(other) {
        return arraysEqual(this.data, other.data);
    }
    getValue(encoding = JSON_ENCODING) {
        if (this._value != undefined) {
            return this._value;
        }
        return encoding.decoder(this.data);
    }
};
__decorate([
    field({ type: Uint8Array }),
    __metadata("design:type", Uint8Array)
], Payload.prototype, "data", void 0);
Payload = __decorate([
    variant(0),
    __metadata("design:paramtypes", [Object])
], Payload);
export { Payload };
let Metadata = class Metadata {
    gid; // graph id
    clock;
    maxChainLength; // longest chain/merkle tree path frmo this node. maxChainLength := max ( maxChainLength(this.next) , 1)
    constructor(properties) {
        if (properties) {
            this.gid = properties.gid;
            this.clock = properties.clock;
            this.maxChainLength = properties.maxChainLength;
        }
    }
};
__decorate([
    field({ type: "string" }),
    __metadata("design:type", String)
], Metadata.prototype, "gid", void 0);
__decorate([
    field({ type: Clock }),
    __metadata("design:type", Clock)
], Metadata.prototype, "clock", void 0);
__decorate([
    field({ type: "u64" }),
    __metadata("design:type", BigInt)
], Metadata.prototype, "maxChainLength", void 0);
Metadata = __decorate([
    variant(0),
    __metadata("design:paramtypes", [Object])
], Metadata);
export { Metadata };
let Signatures = class Signatures {
    signatures;
    constructor(properties) {
        if (properties) {
            this.signatures = properties.signatures;
        }
    }
    equals(other) {
        if (this.signatures.length !== other.signatures.length) {
            return false;
        }
        for (let i = 0; i < this.signatures.length; i++) {
            if (!this.signatures[i].equals(other.signatures[i])) {
                return false;
            }
        }
        return true;
    }
};
__decorate([
    field({ type: vec(MaybeEncrypted) }),
    __metadata("design:type", Array)
], Signatures.prototype, "signatures", void 0);
Signatures = __decorate([
    variant(0),
    __metadata("design:paramtypes", [Object])
], Signatures);
export { Signatures };
let Entry = Entry_1 = class Entry {
    _metadata;
    _payload;
    _next; // Array of hashes (the tree)
    _fork;
    _reserved;
    _signatures;
    hash; // "zd...Foo", we'll set the hash after persisting the entry
    _encryption;
    _encoding;
    createdLocally;
    constructor(obj) {
        this._metadata = obj.metadata;
        this._payload = obj.payload;
        this._signatures = obj.signatures;
        this._next = obj.next;
        this._fork =
            obj.fork ||
                new DecryptedThing({
                    data: serialize(new StringArray({ arr: [] })),
                });
        this._reserved = obj.reserved || [0, 0, 0, 0];
        this.createdLocally = obj.createdLocally;
    }
    init(props) {
        const encryption = props instanceof Entry_1 ? props._encryption : props.encryption;
        this._encryption = encryption;
        this._encoding = props instanceof Entry_1 ? props._encoding : props.encoding;
        return this;
    }
    get encoding() {
        if (!this._encoding) {
            throw new Error("Not initialized");
        }
        return this._encoding;
    }
    get metadata() {
        return this._metadata.decrypted.getValue(Metadata);
    }
    async getMetadata() {
        await this._metadata.decrypt(this._encryption?.getAnyKeypair || (() => Promise.resolve(undefined)));
        return this.metadata;
    }
    get gid() {
        return this.metadata.gid;
    }
    async getGid() {
        return (await this.getMetadata()).gid;
    }
    async getClock() {
        return (await this.getMetadata()).clock;
    }
    get maxChainLength() {
        return this._metadata.decrypted.getValue(Metadata).maxChainLength;
    }
    async getMaxChainLength() {
        return (await this.getMetadata()).maxChainLength;
    }
    get payload() {
        const payload = this._payload.decrypted.getValue(Payload);
        return payload;
    }
    async getPayload() {
        await this._payload.decrypt(this._encryption?.getAnyKeypair || (() => Promise.resolve(undefined)));
        return this.payload;
    }
    async getPayloadValue() {
        const payload = await this.getPayload();
        return payload.getValue(this.encoding);
    }
    get publicKeys() {
        return this.signatures.map((x) => x.publicKey);
    }
    async getPublicKeys() {
        await this.getSignatures();
        return this.publicKeys;
    }
    get next() {
        return this._next.decrypted.getValue(StringArray).arr;
    }
    async getNext() {
        await this._next.decrypt(this._encryption?.getAnyKeypair || (() => Promise.resolve(undefined)));
        return this.next;
    }
    /**
     * Will only return signatures I can decrypt
     * @returns signatures
     */
    get signatures() {
        const signatures = this._signatures.signatures.filter((x) => {
            try {
                x.decrypted;
                return true;
            }
            catch (error) {
                return false;
            }
        }).map((x) => x.decrypted.getValue(SignatureWithKey));
        if (signatures.length === 0) {
            this._signatures?.signatures.forEach((x) => x.clear());
            throw new Error("Failed to resolve any signature");
        }
        return signatures;
    }
    /**
     * Will only return signatures I can decrypt
     * @returns signatures
     */
    async getSignatures() {
        const results = await Promise.allSettled(this._signatures.signatures.map((x) => x.decrypt(this._encryption?.getAnyKeypair || (() => Promise.resolve(undefined)))));
        if (logger.level === "debug" || logger.level === "trace") {
            for (const [i, result] of results.entries()) {
                if (result.status === "rejected") {
                    logger.debug("Failed to decrypt signature with index: " + i);
                }
            }
        }
        return this.signatures;
    }
    /**
     * Will only verify signatures I can decrypt
     * @returns true if all are verified
     */
    async verifySignatures() {
        const signatures = await this.getSignatures();
        if (signatures.length === 0) {
            return false;
        }
        for (const signature of signatures) {
            if (!(await verify(signature.signature, signature.publicKey, Entry_1.toSignable(this)))) {
                return false;
            }
        }
        return true;
    }
    static toSignable(entry) {
        // TODO fix types
        const trimmed = new Entry_1({
            metadata: entry._metadata,
            next: entry._next,
            payload: entry._payload,
            reserved: entry._reserved,
            fork: entry._fork,
            signatures: undefined,
            hash: undefined,
        });
        return serialize(trimmed);
    }
    toSignable() {
        if (this._signatures) {
            throw new Error("Expected signatures to be undefined");
        }
        if (this.hash) {
            throw new Error("Expected hash to be undefined");
        }
        return Entry_1.toSignable(this);
    }
    equals(other) {
        return (arraysEqual(this._reserved, other._reserved) &&
            this._metadata.equals(other._metadata) &&
            this._signatures.equals(other._signatures) &&
            this._next.equals(other._next) &&
            this._fork.equals(other._fork) &&
            this._payload.equals(other._payload)); // dont compare hashes because the hash is a function of the other properties
    }
    async delete(store) {
        if (!this.hash) {
            throw new Error("Missing hash");
        }
        await store.rm(this.hash);
    }
    static createGid(seed) {
        return sha256Base64(seed || randomBytes(32));
    }
    static async create(properties) {
        if (!properties.encoding || !properties.next) {
            properties = {
                ...properties,
                next: properties.next ? properties.next : [],
                encoding: properties.encoding ? properties.encoding : JSON_ENCODING,
            };
        }
        if (!properties.encoding) {
            throw new Error("Missing encoding options");
        }
        if (!isDefined(properties.data))
            throw new Error("Entry requires data");
        if (!isDefined(properties.next) || !Array.isArray(properties.next))
            throw new Error("'next' argument is not an array");
        // Clean the next objects and convert to hashes
        const nexts = properties.next;
        const payloadToSave = new Payload({
            data: properties.encoding.encoder(properties.data),
            value: properties.data,
        });
        const maybeEncrypt = (thing, reciever) => {
            const recievers = reciever
                ? Array.isArray(reciever)
                    ? reciever
                    : [reciever]
                : undefined;
            if (recievers?.length && recievers?.length > 0) {
                if (!properties.encryption) {
                    throw new Error("Encrpryption config not initialized");
                }
                return new DecryptedThing({
                    data: serialize(thing),
                    value: thing,
                }).encrypt(properties.encryption.options.getEncryptionKeypair, ...recievers);
            }
            return new DecryptedThing({
                data: serialize(thing),
                value: thing,
            });
        };
        let clock = properties.clock;
        if (!clock) {
            const hlc = new HLC();
            nexts.forEach((next) => {
                hlc.update(next.metadata.clock.timestamp);
            });
            if (properties.encryption?.reciever.signatures &&
                properties.encryption?.reciever.metadata) {
                throw new Error("Signature is to be encrypted yet the clock is not, which contains the publicKey as id. Either provide a custom Clock value that is not sensitive or set the reciever (encryption target) for the clock");
            }
            clock = new Clock({
                id: new Uint8Array(serialize(properties.identity.publicKey)),
                timestamp: hlc.now(),
            });
        }
        else {
            const cv = clock;
            // check if nexts, that all nexts are happening BEFORE this clock value (else clock make no sense)
            nexts.forEach((n) => {
                if (Timestamp.compare(n.metadata.clock.timestamp, cv.timestamp) >= 0) {
                    throw new Error("Expecting next(s) to happen before entry, got: " +
                        n.metadata.clock.timestamp +
                        " > " +
                        cv.timestamp);
                }
            });
        }
        const payload = await maybeEncrypt(payloadToSave, properties.encryption?.reciever.payload);
        const nextHashes = [];
        let gid;
        let maxChainLength = 0n;
        const maxClock = new Timestamp({ wallTime: 0n, logical: 0 });
        if (nexts?.length > 0) {
            // take min gid as our gid
            nexts.forEach((n) => {
                if (!n.hash) {
                    throw new Error("Expecting hash to be defined to next entries");
                }
                nextHashes.push(n.hash);
                if (maxChainLength < n.maxChainLength ||
                    maxChainLength == n.maxChainLength) {
                    maxChainLength = n.maxChainLength;
                    if (!gid) {
                        gid = n.metadata.gid;
                        return;
                    }
                    // replace gid if next is from alonger chain, or from a later time, or same time but "smaller" gid
                    else if (
                    /*   maxChainLength < n.maxChainLength ||
                      maxClock < n.clock.logical ||
                      (maxClock == n.clock.logical && n.gid < gid) */ // Longest chain?
                    Timestamp.compare(n.metadata.clock.timestamp, maxClock) > 0 ||
                        (Timestamp.compare(n.metadata.clock.timestamp, maxClock) == 0 &&
                            n.metadata.gid < gid)) {
                        gid = n.metadata.gid;
                    }
                }
            });
            if (!gid) {
                throw new Error("Unexpected behaviour, could not find gid");
            }
        }
        else {
            gid = properties.gid || (await Entry_1.createGid(properties.gidSeed));
        }
        maxChainLength += 1n; // include this
        const metadataEncrypted = await maybeEncrypt(new Metadata({
            maxChainLength,
            clock,
            gid,
        }), properties.encryption?.reciever.metadata);
        const next = nextHashes;
        next?.forEach((next) => {
            if (typeof next !== "string") {
                throw new Error("Unsupported next type");
            }
        });
        const nextEncrypted = await maybeEncrypt(new StringArray({
            arr: next,
        }), properties.encryption?.reciever.next);
        const forks = new DecryptedThing({
            data: serialize(new StringArray({ arr: [] })),
        });
        // Sign id, encrypted payload, clock, nexts, refs
        const entry = new Entry_1({
            payload,
            metadata: metadataEncrypted,
            signatures: undefined,
            fork: forks,
            createdLocally: true,
            next: nextEncrypted, // Array of hashes
            /* refs: properties.refs, */
        });
        const signers = properties.signers || [
            async (data) => new SignatureWithKey({
                signature: await properties.identity.sign(data),
                publicKey: properties.identity.publicKey,
            }),
        ];
        const signable = entry.toSignable();
        let signatures = await Promise.all(signers.map((signer) => signer(signable)));
        signatures = signatures.sort((a, b) => arraysCompare(a.signature, b.signature));
        const encryptedSignatures = [];
        const encryptAllSignaturesWithSameKey = isMaybeEryptionPublicKey(properties.encryption?.reciever?.signatures);
        for (const signature of signatures) {
            const encryption = encryptAllSignaturesWithSameKey
                ? properties.encryption?.reciever?.signatures
                : properties.encryption?.reciever?.signatures?.[signature.publicKey.hashcode()];
            const signatureEncrypted = await maybeEncrypt(signature, encryption);
            encryptedSignatures.push(signatureEncrypted);
        }
        entry._signatures = new Signatures({
            signatures: encryptedSignatures,
        });
        entry.init({
            encryption: properties.encryption?.options,
            encoding: properties.encoding,
        });
        if (properties.canAppend) {
            if (!(await properties.canAppend(entry))) {
                throw new AccessError();
            }
        }
        // Append hash and signature
        entry.hash = await Entry_1.toMultihash(properties.store, entry);
        return entry;
    }
    /**
     * Get the multihash of an Entry.
     * @param ipfs An IPFS instance
     * @param entry Entry to get a multihash for
     * @returns {Promise<string>}
     * @example
     * const multfihash = await Entry.toMultihash(ipfs, entry)
     * console.log(multihash)
     * // "Qm...Foo"
     */
    static async toMultihash(store, entry) {
        if (entry.hash) {
            throw new Error("Expected hash to be missing");
        }
        return store.put(await createBlock(serialize(entry), "raw"));
    }
    /**
     * Create an Entry from a hash.
     * @param {IPFS} ipfs An IPFS instance
     * @param {string} hash The hash to create an Entry from
     * @returns {Promise<Entry<T>>}
     * @example
     * const entry = await Entry.fromMultihash(ipfs, "zd...Foo")
     * console.log(entry)
     * // { hash: "Zd...Foo", payload: "hello", next: [] }
     */
    static async fromMultihash(store, hash, options) {
        if (!hash)
            throw new Error(`Invalid hash: ${hash}`);
        const bytes = await store.get(hash, options);
        if (!bytes) {
            throw new Error("Fialed to resolve block: " + hash);
        }
        const entry = deserialize(await getBlockValue(bytes), Entry_1);
        entry.hash = hash;
        return entry;
    }
    /**
     * Compares two entries.
     * @param {Entry} a
     * @param {Entry} b
     * @returns {number} 1 if a is greater, -1 is b is greater
     */
    static compare(a, b) {
        const aClock = a.metadata.clock;
        const bClock = b.metadata.clock;
        const distance = Clock.compare(aClock, bClock);
        if (distance === 0)
            return aClock.id < bClock.id ? -1 : 1;
        return distance;
    }
    /**
     * Check if an entry equals another entry.
     * @param {Entry} a
     * @param {Entry} b
     * @returns {boolean}
     */
    static isEqual(a, b) {
        return a.hash === b.hash;
    }
    /**
     * Check if an entry is a parent to another entry.
     * @param {Entry} entry1 Entry to check
     * @param {Entry} entry2 The parent Entry
     * @returns {boolean}
     */
    static isDirectParent(entry1, entry2) {
        return entry2.next.indexOf(entry1.hash) > -1; // TODO fix types
    }
    /**
     * Find entry's children from an Array of entries.
     * Returns entry's children as an Array up to the last know child.
     * @param {Entry} entry Entry for which to find the parents
     * @param {Array<Entry<T>>} values Entries to search parents from
     * @returns {Array<Entry<T>>}
     */
    static findDirectChildren(entry, values) {
        let stack = [];
        let parent = values.find((e) => Entry_1.isDirectParent(entry, e));
        let prev = entry;
        while (parent) {
            stack.push(parent);
            prev = parent;
            parent = values.find((e) => Entry_1.isDirectParent(prev, e));
        }
        stack = stack.sort((a, b) => Clock.compare(a.metadata.clock, b.metadata.clock));
        return stack;
    }
};
__decorate([
    field({ type: MaybeEncrypted }),
    __metadata("design:type", MaybeEncrypted)
], Entry.prototype, "_metadata", void 0);
__decorate([
    field({ type: MaybeEncrypted }),
    __metadata("design:type", MaybeEncrypted)
], Entry.prototype, "_payload", void 0);
__decorate([
    field({ type: MaybeEncrypted }),
    __metadata("design:type", MaybeEncrypted)
], Entry.prototype, "_next", void 0);
__decorate([
    field({ type: MaybeEncrypted }),
    __metadata("design:type", MaybeEncrypted)
], Entry.prototype, "_fork", void 0);
__decorate([
    field({ type: fixedArray("u8", 4) }),
    __metadata("design:type", Array)
], Entry.prototype, "_reserved", void 0);
__decorate([
    field({ type: option(Signatures) }),
    __metadata("design:type", Signatures)
], Entry.prototype, "_signatures", void 0);
__decorate([
    field({ type: option("string") }) // we do option because we serialize and store this in a block without the hash, to recieve the hash, which we later set
    ,
    __metadata("design:type", String)
], Entry.prototype, "hash", void 0);
Entry = Entry_1 = __decorate([
    variant(0),
    __metadata("design:paramtypes", [Object])
], Entry);
export { Entry };
//# sourceMappingURL=entry.js.map