export class HeadsIndex {
    _index = new Map();
    //  _headsCache?: Entry<T>[];
    _sortFn;
    _gids;
    constructor(properties) {
        this._gids = new Map();
        this._sortFn = properties.sortFn;
        this.reset(properties.entries);
    }
    get index() {
        return this._index;
    }
    get gids() {
        return this._gids;
    }
    reset(entries) {
        this._index.clear();
        this._gids = new Map();
        entries.forEach((v) => {
            this.put(v);
        });
    }
    get(hash) {
        return this._index.get(hash);
    }
    put(entry) {
        if (!entry.hash) {
            throw new Error("Missing hash");
        }
        if (this._index.has(entry.hash)) {
            return;
        }
        this._index.set(entry.hash, entry);
        if (!this._gids.has(entry.gid)) {
            this._gids.set(entry.gid, 1);
        }
        else {
            this._gids.set(entry.gid, this._gids.get(entry.gid) + 1);
        }
    }
    del(entry) {
        const wasHead = this._index.delete(entry.hash);
        if (!wasHead) {
            return {
                lastWithGid: false,
                removed: false,
            };
        }
        const newValue = this._gids.get(entry.gid) - 1;
        const lastWithGid = newValue <= 0;
        if (newValue <= 0) {
            this._gids.delete(entry.gid);
        }
        else {
            this._gids.set(entry.gid, newValue);
        }
        if (!entry.hash) {
            throw new Error("Missing hash");
        }
        return {
            removed: wasHead,
            lastWithGid: lastWithGid,
        };
        //     this._headsCache = undefined; // TODO do smarter things here, only remove the element needed (?)
    }
    /**
     * Returns an array of heads.
     * Dont use this anywhere where performance matters
     * @returns {Array<Entry<T>>}
     */
    get array() {
        return [...this._index.values()].sort(this._sortFn).reverse();
    }
}
//# sourceMappingURL=heads.js.map