import { Entry } from "./entry.js";
import { EntryFetchAllOptions } from "./entry-io.js";
import { ISortFunction } from "./log-sorting.js";
import { BlockStore } from "@dao-xyz/libp2p-direct-block";
import { Log } from "./log.js";
export declare class LogIO {
    /**
     * Get the multihash of a Log.
     * @param {IPFS} ipfs An IPFS instance
     * @param {Log} log Log to get a multihash for
     * @returns {Promise<string>}
     * @deprecated
     */
    static toMultihash(store: BlockStore, log: Log<any>, options?: {
        format?: string;
    }): Promise<string>;
    static fromMultihash<T>(store: BlockStore, hash: string, options: EntryFetchAllOptions<T> & {
        sortFn: any;
    }): Promise<{
        logId: string;
        entries: any[];
        heads: any[];
    }>;
    static fromEntryHash<T>(store: BlockStore, hash: string[] | string, options: EntryFetchAllOptions<T> & {
        sortFn?: ISortFunction;
    }): Promise<{
        entries: any[];
    }>;
    /**
     * Creates a log data from a JSON object, to be passed to a Log constructor
     *
     * @param {IPFS} ipfs An IPFS instance
     * @param {json} json A json object containing valid log data
     * @param {Object} options
     * @param {number} options.length How many entries to include
     * @param {function(hash, entry,  parent, depth)} options.onFetched
     **/
    static fromJSON<T>(store: BlockStore, json: {
        id: string;
        heads: string[];
    }, options: EntryFetchAllOptions<T>): Promise<{
        logId: string;
        entries: Entry<T>[];
        heads: string[];
    }>;
    static fromEntry<T>(store: BlockStore, sourceEntries: Entry<T>[] | Entry<T>, options: EntryFetchAllOptions<T>): Promise<{
        entries: Entry<T>[];
    }>;
}
